"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""
from {{app}}.model.sample_user import User


def make(user:User)->dict:
    """
    Show information only client can see
    :param user:
    :return:
    """
    return {
        'name': user.name,
        'age': user.age,
    }


"""
집합관련 유틸
"""


class HashableDict(dict):
    # hash 로 쓸 dict 의 field 를 지정할 수 있음
    def __init__(self, dict_object, keys_to_hash):
        self.__keys_to_hash = keys_to_hash
        super().__init__(dict_object)

    def __hash__(self):
        return hash(self._make_hash_str())

    def __eq__(self, other):
        return self.__hash__() == other.__hash__()

    def _make_hash_str(self):
        return '|'.join([str(self.get(key, '')) for key in self.__keys_to_hash])


if __name__ == '__main__':
    d1 = {1: 2, 3: 5}
    d2 = {1: 2, 4: 7}
    d1 = HashableDict(d1, [1])
    d2 = HashableDict(d2, [2])
    s = {d1, d2}
    print(s)


"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""

import time
import uuid
import api
from pydantic import fields
from typing import Optional, Literal, TypeVar, Generic, Type
from sawsi.model.base import DynamoModel


class Address(DynamoModel):
    __dynamo__ = api.dynamo
    _table = 'address'
    _indexes = [
        ('street', 'city'),
        ('street', 'country')
    ]

    street: str
    city: str
    country: str


class User(DynamoModel):
    __dynamo__ = api.dynamo
    _table = 'user'
    _indexes = [
        ('gender', 'crt'),
    ]

    id: str = fields.Field(default_factory=lambda: str(uuid.uuid4()))
    crt:float = fields.Field(default_factory=time.time)
    ptn:str = 'user'
    name: str
    age: int
    gender: Optional[Literal["male", "female"]] = None
    address: Address


if __name__ == '__main__':
    # 모델 인스턴스 생성
    address_data = {'street': '123 Main St', 'city': 'Anytown', 'country': 'USA'}
    user_data = {'name': 'John Doe', 'age': 30, 'gender': 'male', 'address': address_data}
    user = User(**user_data)
    print('user.id', user.id)

    # 모델을 딕셔너리로 변환
    user_dict = user.model_dump()
    print(user_dict)



from sawsi.aws.secrets_manager.api import SecretManagerAPI
from sawsi.aws.dynamodb.api import DynamoDBAPI
from sawsi.aws.s3.api import S3API
from sawsi.aws.locking.api import LockingAPI
from sawsi.aws.firehose.api import Firehose
from sawsi.aws.ses.api import SESAPI
from sawsi.aws.sqs.api import SQSAPI
import config


env = config.env
region = config.region


if not env:
    raise Exception('os.getenv() is None')


"""
Bellow codes are sample
You can modify or delete
"""

project_name = '{{project}}'

secret_manager: SecretManagerAPI = SecretManagerAPI(f'{env}/{project_name}', region=region)
dynamo = DynamoDBAPI(f'{env}-{project_name}', region=region)
s3_public = S3API(f'{env}-{project_name}-public', region=region)

locking = LockingAPI(f'{env}-{project_name}-locking', region=region)
firehose_log = Firehose(f'{env}-{project_name}', f'{env}-{project_name}-firehose', 'log', region=region)

ses = SESAPI(region=region)
sqs = SQSAPI(f'https://sqs.{region}.amazonaws.com/<aws_account_id>/{env}-queue', region=region)
